==> katana_routing-0.8.4%2B2.tar.gz: lib/src/ui_scope.dart <==
part of katana_routing;

/// You can limit the rebuild range by tucking it inside a widget.
///
/// Basically, it is used in the same way as [Consumer].
///
/// ```
/// UIScope(
///   builder: (context, watch) {
///     final document = watch(localDocumentProvider("local"));
///     context.rebuild();
///   }
/// )
/// ```
class UIScope extends StatefulWidget {
  /// You can limit the rebuild range by tucking it inside a widget.
  ///
  /// Basically, it is used in the same way as [Consumer].
  ///
  /// ```
  /// UIScope(
  ///   builder: (context, watch) {
  ///     final document = watch(localDocumentProvider("local"));
  ///     context.rebuild();
  ///   }
  /// )
  /// ```
  const UIScope({Key? key, required this.builder}) : super(key: key);

  /// Builder of Scope.
  final Widget Function(
          BuildContext context, T Function<T>(ProviderBase<Object?, T>) watch)
      builder;

  /// The closest instance of this class that encloses the given context.
  ///
  /// Typical usage:
  ///
  /// ```dart
  /// final state = UIScope.of(context);
  /// ```
  static UIScopeStateBase of(BuildContext context) {
    final scope = context
        .getElementForInheritedWidgetOfExactType<_UIScope>()!
        .widget as _UIScope;
    return scope.state;
  }

  /// Creates the mutable state for this widget at a given location in the tree.
  ///
  /// Subclasses should override this method to return a newly created
  /// instance of their associated [State] subclass:
  ///
  /// ```dart
  /// @override
  /// _MyState createState() => _MyState();
  /// ```
  ///
  /// The framework can call this method multiple times over the lifetime of
  /// a [StatefulWidget]. For example, if the widget is inserted into the tree
  /// in multiple locations, the framework will create a separate [State] object
  /// for each location. Similarly, if the widget is removed from the tree and
  /// later inserted into the tree again, the framework will call [createState]
  /// again to create a fresh [State] object, simplifying the lifecycle of
  /// [State] objects.
  @override
  @protected
  State<StatefulWidget> createState() => UIScopeState();
}

class UIScopeState extends State<UIScope> implements UIScopeStateBase {
  @override
  final DynamicMap _map = {};

  /// Explicitly rebuild the widget.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  @override
  void rebuild() => setState(() {});

  /// Rebuild all active pages.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  @override
  void rebuildAll() {
    PageState._pageStack.forEach((element) => element.rebuild());
  }

  /// Rebuild only the pages that are `true` in [test].
  ///
  /// Normally, you should check if you want to rebuild based on the page name.
  /// ```
  /// context.rebuildWhere((page) {
  ///   return page.routeSettings?.name == "/";
  /// });
  /// ```
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  @override
  void rebuildWhere(bool Function(PageState page) test) {
    PageState._pageStack.where(test).forEach((element) => element.rebuild());
  }

  /// Describes the part of the user interface represented by this widget.
  ///
  /// The framework calls this method in a number of different situations. For
  /// example:
  ///
  ///  * After calling [initState].
  ///  * After calling [didUpdateWidget].
  ///  * After receiving a call to [setState].
  ///  * After a dependency of this [State] object changes (e.g., an
  ///    [InheritedWidget] referenced by the previous [build] changes).
  ///  * After calling [deactivate] and then reinserting the [State] object into
  ///    the tree at another location.
  ///
  /// This method can potentially be called in every frame and should not have
  /// any side effects beyond building a widget.
  ///
  /// The framework replaces the subtree below this widget with the widget
  /// returned by this method, either by updating the existing subtree or by
  /// removing the subtree and inflating a new subtree, depending on whether the
  /// widget returned by this method can update the root of the existing
  /// subtree, as determined by calling [Widget.canUpdate].
  ///
  /// Typically implementations return a newly created constellation of widgets
  /// that are configured with information from this widget's constructor, the
  /// given [BuildContext], and the internal state of this [State] object.
  ///
  /// The given [BuildContext] contains information about the location in the
  /// tree at which this widget is being built. For example, the context
  /// provides the set of inherited widgets for this location in the tree. The
  /// [BuildContext] argument is always the same as the [context] property of
  /// this [State] object and will remain the same for the lifetime of this
  /// object. The [BuildContext] argument is provided redundantly here so that
  /// this method matches the signature for a [WidgetBuilder].
  ///
  /// ## Design discussion
  ///
  /// ### Why is the [build] method on [State], and not [StatefulWidget]?
  ///
  /// Putting a `Widget build(BuildContext context)` method on [State] rather
  /// than putting a `Widget build(BuildContext context, State state)` method
  /// on [StatefulWidget] gives developers more flexibility when subclassing
  /// [StatefulWidget].
  ///
  /// For example, [AnimatedWidget] is a subclass of [StatefulWidget] that
  /// introduces an abstract `Widget build(BuildContext context)` method for its
  /// subclasses to implement. If [StatefulWidget] already had a [build] method
  /// that took a [State] argument, [AnimatedWidget] would be forced to provide
  /// its [State] object to subclasses even though its [State] object is an
  /// internal implementation detail of [AnimatedWidget].
  ///
  /// Conceptually, [StatelessWidget] could also be implemented as a subclass of
  /// [StatefulWidget] in a similar manner. If the [build] method were on
  /// [StatefulWidget] rather than [State], that would not be possible anymore.
  ///
  /// Putting the [build] function on [State] rather than [StatefulWidget] also
  /// helps avoid a category of bugs related to closures implicitly capturing
  /// `this`. If you defined a closure in a [build] function on a
  /// [StatefulWidget], that closure would implicitly capture `this`, which is
  /// the current widget instance, and would have the (immutable) fields of that
  /// instance in scope:
  ///
  /// ```dart
  /// class MyButton extends StatefulWidget {
  ///   ...
  ///   final Color color;
  ///
  ///   @override
  ///   Widget build(BuildContext context, MyButtonState state) {
  ///     ... () { print("color: $color"); } ...
  ///   }
  /// }
  /// ```
  ///
  /// For example, suppose the parent builds `MyButton` with `color` being blue,
  /// the `$color` in the print function refers to blue, as expected. Now,
  /// suppose the parent rebuilds `MyButton` with green. The closure created by
  /// the first build still implicitly refers to the original widget and the
  /// `$color` still prints blue even through the widget has been updated to
  /// green.
  ///
  /// In contrast, with the [build] function on the [State] object, closures
  /// created during [build] implicitly capture the [State] instance instead of
  /// the widget instance:
  ///
  /// ```dart
  /// class MyButtonState extends State<MyButton> {
  ///   ...
  ///   @override
  ///   Widget build(BuildContext context) {
  ///     ... () { print("color: ${widget.color}"); } ...
  ///   }
  /// }
  /// ```
  ///
  /// Now when the parent rebuilds `MyButton` with green, the closure created by
  /// the first build still refers to [State] object, which is preserved across
  /// rebuilds, but the framework has updated that [State] object's [widget]
  /// property to refer to the new `MyButton` instance and `${widget.color}`
  /// prints green, as expected.
  ///
  /// See also:
  ///
  ///  * [StatefulWidget], which contains the discussion on performance considerations.
  @override
  Widget build(BuildContext context) {
    return _UIScope(
      state: this,
      child: Consumer(
        key: widget.key,
        builder: (context, watch, child) {
          return widget.builder(context, watch);
        },
      ),
    );
  }
}

/// Base class for UIScope.ｓ
abstract class UIScopeStateBase {
  /// Explicitly rebuild the widget.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuild();

  /// Rebuild all active pages.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuildAll();

  /// Rebuild only the pages that are `true` in [test].
  ///
  /// Normally, you should check if you want to rebuild based on the page name.
  /// ```
  /// context.rebuildWhere((page) {
  ///   return page.routeSettings?.name == "/";
  /// });
  /// ```
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuildWhere(bool Function(PageState page) test);
  DynamicMap get _map;
}

class _UIScope extends InheritedWidget {
  const _UIScope({required Widget child, required this.state, Key? key})
      : super(key: key, child: child);
  final UIScopeStateBase state;
  @override
  bool updateShouldNotify(_UIScope old) {
    return true;
  }
}

==> katana_routing-0.8.5%2B2.tar.gz: lib/src/ui_scope.dart <==
part of katana_routing;

/// You can limit the rebuild range by tucking it inside a widget.
///
/// Basically, it is used in the same way as [Consumer].
///
/// ```
/// UIScope(
///   builder: (context, ref) {
///     final document = ref.watch(localDocumentProvider("local"));
///     context.rebuild();
///   }
/// )
/// ```
class UIScope extends StatefulWidget {
  /// You can limit the rebuild range by tucking it inside a widget.
  ///
  /// Basically, it is used in the same way as [Consumer].
  ///
  /// ```
  /// UIScope(
  ///   builder: (context, ref) {
  ///     final document = ref.watch(localDocumentProvider("local"));
  ///     context.rebuild();
  ///   }
  /// )
  /// ```
  const UIScope({Key? key, required this.builder}) : super(key: key);

  /// Builder of Scope.
  final Widget Function(BuildContext context, WidgetRef ref) builder;

  /// The closest instance of this class that encloses the given context.
  ///
  /// Typical usage:
  ///
  /// ```dart
  /// final state = UIScope.of(context);
  /// ```
  static UIScopeStateBase of(BuildContext context) {
    final scope = context
        .getElementForInheritedWidgetOfExactType<_UIScope>()!
        .widget as _UIScope;
    return scope.state;
  }

  /// Creates the mutable state for this widget at a given location in the tree.
  ///
  /// Subclasses should override this method to return a newly created
  /// instance of their associated [State] subclass:
  ///
  /// ```dart
  /// @override
  /// _MyState createState() => _MyState();
  /// ```
  ///
  /// The framework can call this method multiple times over the lifetime of
  /// a [StatefulWidget]. For example, if the widget is inserted into the tree
  /// in multiple locations, the framework will create a separate [State] object
  /// for each location. Similarly, if the widget is removed from the tree and
  /// later inserted into the tree again, the framework will call [createState]
  /// again to create a fresh [State] object, simplifying the lifecycle of
  /// [State] objects.
  @override
  @protected
  State<StatefulWidget> createState() => UIScopeState();
}

class UIScopeState extends State<UIScope> implements UIScopeStateBase {
  @override
  final DynamicMap _map = {};

  /// Explicitly rebuild the widget.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  @override
  void rebuild() => setState(() {});

  /// Rebuild all active pages.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  @override
  void rebuildAll() {
    PageState._pageStack.forEach((element) => element.rebuild());
  }

  /// Rebuild only the pages that are `true` in [test].
  ///
  /// Normally, you should check if you want to rebuild based on the page name.
  /// ```
  /// context.rebuildWhere((page) {
  ///   return page.routeSettings?.name == "/";
  /// });
  /// ```
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  @override
  void rebuildWhere(bool Function(PageState page) test) {
    PageState._pageStack.where(test).forEach((element) => element.rebuild());
  }

  /// Describes the part of the user interface represented by this widget.
  ///
  /// The framework calls this method in a number of different situations. For
  /// example:
  ///
  ///  * After calling [initState].
  ///  * After calling [didUpdateWidget].
  ///  * After receiving a call to [setState].
  ///  * After a dependency of this [State] object changes (e.g., an
  ///    [InheritedWidget] referenced by the previous [build] changes).
  ///  * After calling [deactivate] and then reinserting the [State] object into
  ///    the tree at another location.
  ///
  /// This method can potentially be called in every frame and should not have
  /// any side effects beyond building a widget.
  ///
  /// The framework replaces the subtree below this widget with the widget
  /// returned by this method, either by updating the existing subtree or by
  /// removing the subtree and inflating a new subtree, depending on whether the
  /// widget returned by this method can update the root of the existing
  /// subtree, as determined by calling [Widget.canUpdate].
  ///
  /// Typically implementations return a newly created constellation of widgets
  /// that are configured with information from this widget's constructor, the
  /// given [BuildContext], and the internal state of this [State] object.
  ///
  /// The given [BuildContext] contains information about the location in the
  /// tree at which this widget is being built. For example, the context
  /// provides the set of inherited widgets for this location in the tree. The
  /// [BuildContext] argument is always the same as the [context] property of
  /// this [State] object and will remain the same for the lifetime of this
  /// object. The [BuildContext] argument is provided redundantly here so that
  /// this method matches the signature for a [WidgetBuilder].
  ///
  /// ## Design discussion
  ///
  /// ### Why is the [build] method on [State], and not [StatefulWidget]?
  ///
  /// Putting a `Widget build(BuildContext context)` method on [State] rather
  /// than putting a `Widget build(BuildContext context, State state)` method
  /// on [StatefulWidget] gives developers more flexibility when subclassing
  /// [StatefulWidget].
  ///
  /// For example, [AnimatedWidget] is a subclass of [StatefulWidget] that
  /// introduces an abstract `Widget build(BuildContext context)` method for its
  /// subclasses to implement. If [StatefulWidget] already had a [build] method
  /// that took a [State] argument, [AnimatedWidget] would be forced to provide
  /// its [State] object to subclasses even though its [State] object is an
  /// internal implementation detail of [AnimatedWidget].
  ///
  /// Conceptually, [StatelessWidget] could also be implemented as a subclass of
  /// [StatefulWidget] in a similar manner. If the [build] method were on
  /// [StatefulWidget] rather than [State], that would not be possible anymore.
  ///
  /// Putting the [build] function on [State] rather than [StatefulWidget] also
  /// helps avoid a category of bugs related to closures implicitly capturing
  /// `this`. If you defined a closure in a [build] function on a
  /// [StatefulWidget], that closure would implicitly capture `this`, which is
  /// the current widget instance, and would have the (immutable) fields of that
  /// instance in scope:
  ///
  /// ```dart
  /// class MyButton extends StatefulWidget {
  ///   ...
  ///   final Color color;
  ///
  ///   @override
  ///   Widget build(BuildContext context, MyButtonState state) {
  ///     ... () { print("color: $color"); } ...
  ///   }
  /// }
  /// ```
  ///
  /// For example, suppose the parent builds `MyButton` with `color` being blue,
  /// the `$color` in the print function refers to blue, as expected. Now,
  /// suppose the parent rebuilds `MyButton` with green. The closure created by
  /// the first build still implicitly refers to the original widget and the
  /// `$color` still prints blue even through the widget has been updated to
  /// green.
  ///
  /// In contrast, with the [build] function on the [State] object, closures
  /// created during [build] implicitly capture the [State] instance instead of
  /// the widget instance:
  ///
  /// ```dart
  /// class MyButtonState extends State<MyButton> {
  ///   ...
  ///   @override
  ///   Widget build(BuildContext context) {
  ///     ... () { print("color: ${widget.color}"); } ...
  ///   }
  /// }
  /// ```
  ///
  /// Now when the parent rebuilds `MyButton` with green, the closure created by
  /// the first build still refers to [State] object, which is preserved across
  /// rebuilds, but the framework has updated that [State] object's [widget]
  /// property to refer to the new `MyButton` instance and `${widget.color}`
  /// prints green, as expected.
  ///
  /// See also:
  ///
  ///  * [StatefulWidget], which contains the discussion on performance considerations.
  @override
  Widget build(BuildContext context) {
    return _UIScope(
      state: this,
      child: Consumer(
        key: widget.key,
        builder: (context, ref, child) {
          return widget.builder(context, ref);
        },
      ),
    );
  }
}

/// Base class for UIScope.ｓ
abstract class UIScopeStateBase {
  DynamicMap get _map;

  /// The location in the tree where this widget builds.
  ///
  /// The framework associates [State] objects with a [BuildContext] after
  /// creating them with [StatefulWidget.createState] and before calling
  /// [initState]. The association is permanent: the [State] object will never
  /// change its [BuildContext]. However, the [BuildContext] itself can be moved
  /// around the tree.
  ///
  /// After calling [dispose], the framework severs the [State] object's
  /// connection with the [BuildContext].
  BuildContext get context;

  /// Explicitly rebuild the widget.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuild();

  /// Rebuild all active pages.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuildAll();

  /// Rebuild only the pages that are `true` in [test].
  ///
  /// Normally, you should check if you want to rebuild based on the page name.
  /// ```
  /// context.rebuildWhere((page) {
  ///   return page.routeSettings?.name == "/";
  /// });
  /// ```
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuildWhere(bool Function(PageState page) test);
}

class _UIScope extends InheritedWidget {
  const _UIScope({required Widget child, required this.state, Key? key})
      : super(key: key, child: child);
  final UIScopeStateBase state;
  @override
  bool updateShouldNotify(_UIScope old) {
    return true;
  }
}
